@@ -1,6 +1,6 @@
 /*
   Licensed under the GNU Public License.
-  Copyright (C) 1998-2001 by Thomas M. Vier, Jr. All Rights Reserved.
+  Copyright (C) 1998-2003 by Thomas M. Vier, Jr. All Rights Reserved.
 
   wipe is free software.
   See LICENSE for more information.
@@ -23,8 +23,11 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <errno.h>
+#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <fcntl.h>
+#include <dirent.h>
 
 #define __USE_GNU
 #include <string.h>
@@ -40,6 +43,7 @@
 #include "main.h"
 #include "percent.h"
 #include "file.h"
+#include "io.h"
 #include "dir.h"
 
 extern int errno;
@@ -57,13 +61,13 @@ public void drill_down(const char str[])
 {
   DIR *dir;
   struct dirent *entry;
-  char *base, dirname[PATH_MAX+1];
+  char *base, real_base[PATH_MAX+1];
+  char dirname[PATH_MAX+1];
   size_t len;
   char prompt[4];
 
-  /* copy str to dirname, so we can write to it */
   strncpy(dirname, str, PATH_MAX);
-  dirname[sizeof(dirname)-1] = 0; /* term str */
+  dirname[sizeof(dirname)-1] = 0;
 
 #ifdef DEBUG
   fprintf(stderr, "checking %s\n", dirname);
@@ -80,7 +84,7 @@ public void drill_down(const char str[])
     {
       fprintf(stderr, "\r%s: cannot enter directory `%s': %s\n",
 	      argvzero, dirname, strerror(errno));
-      closedir(dir); exit_code = errno; return;
+      exit_code = errno; goto cleanup;
     }
 
   while ((entry = readdir(dir)) != NULL)
@@ -96,8 +100,6 @@ public void drill_down(const char str[])
       abort();
     }
 
-  closedir(dir);
-
 #ifdef DEBUG
   if (options.delete)
     fprintf(stderr, "\rwould have removed: %s\n", dirname);
@@ -135,41 +137,54 @@ public void drill_down(const char str[])
 
       if (options.interactive)
 	{
-	  prompt[0] = 0; /* clear prompt */
+	  prompt[0] = 0;
 	  while (prompt[0] != 'y' && prompt[0] != 'n')
 	    {
 	      printf("\r%s: remove directory `%s'? ",
 		     argvzero, dirname);
 	      fgets(prompt, sizeof(prompt), stdin);
 
-	      if (prompt[0] == 'y' || prompt[0] == 'Y') goto removedir;
-	      if (prompt[0] == 'n' || prompt[0] == 'N') return; /* skip to next file */
-
-	      /* else, incorrect response -- re-prompt */
-	    } /* while prompt */
+	      if (prompt[0] == 'y' || prompt[0] == 'Y') break;
+	      if (prompt[0] == 'n' || prompt[0] == 'N') goto cleanup;
+	    }
 	}
 
-    removedir:
-      if (rmdir(base))
+      strncpy(real_base, base, PATH_MAX);
+      real_base[sizeof(real_base)-1] = 0;
+
+      wipe_name(real_base, strnlen(real_base, sizeof(real_base)));
+      wipe_name(real_base, 0);
+
+      if (rmdir(real_base))
 	{
 	  fprintf(stderr, "\r%s: %s: unable to remove directory: `%s'\n",
 		  argvzero, strerror(errno), base);
-	  exit_code = errno; return;
+	  exit_code = errno;
 	}
     }
+
+ cleanup:
+  if (closedir(dir))
+    {
+      fprintf(stderr, "\r%s: closedir failed for `%s': %s\n",
+	      argvzero, dirname, strerror(errno));
+      exit_code = errno; return;
+    }
 #endif
 }
 
 /*
   wipe_name -- rename to random characters
 
-  there's not much sense in doing the wipe passes
-  on the filename. let's just give it a random name.
+  rnlen - length to overwrite, or 0 for don't care (NAME_MAX)
+
+  this won't work for hashed dirs
 */
 
-public int wipe_name(struct file_s *f)
+public int wipe_name(char *real_name, size_t rnlen)
 {
-  int i=0;
+  DIR *dir;
+  int i, fd;
   size_t len, pathlen;
   char *base, dest_name[PATH_MAX+1];
 
@@ -182,22 +197,27 @@ public int wipe_name(struct file_s *f)
     }
 #endif
 
-  /* make sure it's NUL-terminated */
-  dest_name[sizeof(dest_name)-1] = 0;
+  if (rnlen == 0 || rnlen > NAME_MAX)
+    rnlen = NAME_MAX;
+
+  if (rnlen > 0) ++rnlen;
 
   /* copy the path to the file */
 
   /*
-    strncpy does NOT guarantee NUL termination - drill that into your head
+    strncpy does NOT guarantee NUL termination
     it's an often over looked hole for a buffer overflow exploit from
     untrusted input.
   */
-  strncpy(dest_name, f->real_name, PATH_MAX);
-
-  f->real_name[sizeof(f->real_name)-1] = 0;
+  strncpy(dest_name, real_name, rnlen);
+  dest_name[sizeof(dest_name)-1] = 0;
 
   /* point base at just after the path, ie, strip the path */
-  if ((base = strrchr(dest_name, '/')) == NULL)
+  base = strrchr(dest_name, '/');
+
+  /* strrchr doesn't ask for the boundry */
+  if (base == NULL // not found
+      || base >= (dest_name + sizeof(dest_name))) // boundry check
     base = dest_name;
   else
     ++base;
@@ -205,40 +225,84 @@ public int wipe_name(struct file_s *f)
   /* truncate the path to get the length */
   *base = 0x00;
 
-  pathlen = strnlen(dest_name, PATH_MAX);
+  pathlen = strnlen(dest_name, sizeof(dest_name));
+
+  /*
+    overwrite the filename, same length
+    won't work for hashed dirs
+
+    the second time wipe_name() called, it'll be trying to
+    overwrite the rand name used last time
+  */
+
+  //len = strnlen((strrchr(real_name, '/')+1));
 
   /*
     we try to use as long a filename as possible,
     but not longer than PATH_MAX.
   */
-  len = NAME_MAX;
-  while (len + pathlen > PATH_MAX) --len;
+
+  len = pathlen + rnlen;
+  if (len > PATH_MAX)
+    rnlen -= len - PATH_MAX;
 
   /*
     get a random filename, but make sure that there
     isn't one with the same name prior renaming --tg
   */
 
+  i=0;
   while (!i)
     {
-      rename_str(base, len);
+      rename_str(base, rnlen);
       i = access(dest_name, F_OK);
     }
 
 #ifdef NORENAME
   /* debugging aid */
-  fprintf(stderr, "\rrenamed to %s\n", base);
+  fprintf(stderr, "\rwould have renamed to %s\n", base);
   return 0;
 #endif
 
-  if (rename(f->real_name, dest_name) == 0)
+  if (rename(real_name, dest_name) == 0)
     {
       /* update pathname */
-      strncpy(f->real_name, dest_name, PATH_MAX);
-      f->real_name[sizeof(f->real_name)-1] = 0;
+      strncpy(real_name, dest_name, rnlen);
+      real_name[rnlen-1] = 0;
     }
   else
-    return FAILED;
+    {
+      fprintf(stderr, "\r%s: cannot rename `%s': %s\n",
+	      argvzero, real_name, strerror(errno));
+      exit_code = errno; return FAILED;
+    }
+
+#ifdef HAVE_DIRFD
+  /* sync the dir */
+  if ((dir = opendir(".")) == NULL)
+    {
+      fprintf(stderr, "\r%s: cannot open directory `.': %s\n",
+	      argvzero, strerror(errno));
+      exit_code = errno; return FAILED;
+    }
+
+  if ((fd = dirfd(dir)) < 0)
+    {
+      fprintf(stderr, "\r%s: dirfd() failed for `.': %s\n",
+	      argvzero, strerror(errno));
+      exit_code = errno; return FAILED;
+    }
+
+  do_fsync(".", fd);
+
+  /* closedir() also closes the fd - not mentioned in man page */
+  if (closedir(dir))
+    {
+      fprintf(stderr, "\r%s: closedir failed for `.': %s\n",
+	      argvzero, strerror(errno));
+      exit_code = errno; return FAILED;
+    }
+#endif
 
   return 0;
 }
