@@ -1,6 +1,6 @@
 /*
   Licensed under the GNU Public License.
-  Copyright (C) 1998-2001 by Thomas M. Vier, Jr. All Rights Reserved.
+  Copyright (C) 1998-2003 by Thomas M. Vier, Jr. All Rights Reserved.
 
   wipe is free software.
   See LICENSE for more information.
@@ -21,4 +21,4 @@
 */
 
 public void drill_down(const char str[]);
-public int wipe_name(struct file_s *f);
+public int wipe_name(char *real_name, size_t rnlen);
