@@ -1,6 +1,6 @@
 /*
   Licensed under the GNU Public License.
-  Copyright (C) 1998-2002 by Thomas M. Vier, Jr. All Rights Reserved.
+  Copyright (C) 1998-2003 by Thomas M. Vier, Jr. All Rights Reserved.
 
   wipe is free software.
   See LICENSE for more information.
@@ -41,6 +41,7 @@
 #include "file.h"
 #include "rand.h"
 #include "prompt.h"
+#include "wipe.h"
 
 extern int errno;
 extern int exit_code;
@@ -62,10 +63,14 @@ public void do_file(const char *name)
   if ((code = fgetdata(name, &f)) == FAILED)
     {exit_code = code; return;}
 
+  if (options.no_file)
+    {destroy_file(&f); return;}
+
   switch (f.st.st_mode & S_IFMT)
     {
     case S_IFREG: /* regular file */
     case S_IFBLK: /* block device */
+    case S_IFCHR: /* char dev */
       prompt_destroy(&f, R_OK|W_OK);
       break;
 
@@ -73,10 +78,6 @@ public void do_file(const char *name)
       prompt_recursion(name, R_OK|W_OK|X_OK, f.st.st_mode);
       break;
 
-    case S_IFCHR: /* char dev */
-      prompt_unlink(name, "character device file", W_OK, f.st.st_mode);
-      return;
-
     case S_IFIFO: /* fifo */
       prompt_unlink(name, "fifo", W_OK, f.st.st_mode);
       return;
@@ -88,6 +89,7 @@ public void do_file(const char *name)
     case S_IFLNK: /* sym link */
       prompt_unlink(name, "symbolic link", W_OK, f.st.st_mode);
       return;
+
     default:
       abort();
     }
@@ -117,15 +119,20 @@ public void fsetbuf(struct file_s *f)
      have to worry about f->bufsize % 3 == 0
   */
 
-  if (!S_ISBLK(f->st.st_mode))
+  if (!options.no_file)
     {
-      f->fsize = f->st.st_size;
+      if (!S_ISBLK(f->st.st_mode))
+	{
+	  f->fsize = f->st.st_size;
 
-      if (f->fsize % f->st.st_blksize)
-	f->fsize += f->st.st_blksize - (f->fsize % f->st.st_blksize);
-      else
-	++f->fsize;
+	  if (f->fsize % f->st.st_blksize)
+	    f->fsize += f->st.st_blksize - (f->fsize % f->st.st_blksize);
+	  else
+	    ++f->fsize;
+	}
     }
+  else
+    f->fsize = options.stdout_size;
 
   /*
     note that i don't do 
@@ -148,7 +155,7 @@ public void fsetbuf(struct file_s *f)
   else
     {
       f->loop = 1; f->loopr = 0;
-      f->bufsize = f->fsize;
+      if (f->fsize) f->bufsize = f->fsize;
     }
 }
 
@@ -175,10 +182,10 @@ public int fgetbuf(struct file_s *f)
     }
 
 #ifdef SANITY
-  if ((unsigned long int) f->buf % sizeof(urand_t))
+  if ((unsigned long int) f->buf % sizeof(u_rand_t))
     {
       fprintf(stderr, "\r%s: buffer not %ld byte aligned! Skipping `%s'\n",
-	      argvzero, (long int) sizeof(urand_t), f->name);
+	      argvzero, (long int) sizeof(u_rand_t), f->name);
       return FAILED;
     }
 #endif
@@ -208,16 +215,23 @@ private int fgetdata(const char *name, struct file_s *f)
   f->name[sizeof(f->name)-1] = 0;
   f->real_name[sizeof(f->real_name)-1] = 0;
 
-  /* get inode data */
-  if (lstat(f->name, &f->st))
+  if (!options.no_file)
     {
-      fprintf(stderr, "\r%s: cannot stat `%s': %s\n",
-	      argvzero, f->name, strerror(errno));
-      return FAILED;
+      /* get inode data */
+      if (lstat(f->name, &f->st))
+	{
+	  fprintf(stderr, "\r%s: cannot stat `%s': %s\n",
+		  argvzero, f->name, strerror(errno));
+	  return FAILED;
+	}
     }
 
   /* init to safe values */
-  f->fd = -1;
+  if (options.no_file)
+    f->fd = 1;
+  else
+    f->fd = -1;
+
   f->buf = NULL;
   f->bufsize=0; f->loop=0;
 
