@@ -1,6 +1,6 @@
 /*
   Licensed under the GNU Public License.
-  Copyright (C) 1998-2001 by Thomas M. Vier, Jr. All Rights Reserved.
+  Copyright (C) 1998-2003 by Thomas M. Vier, Jr. All Rights Reserved.
 
   wipe is free software.
   See LICENSE for more information.
@@ -165,7 +165,18 @@ public int do_write(const char name[], const int fd, void *buf, size_t count)
   if (c == -1)
     {
       if (errno == ENOSPC && options.until_full)
-	return ENOSPC;
+	{
+	  /* write as much as possible */
+	  //do_write(name, fd, buf, count-1);
+	  int i;
+	  for (i=count-1; i > 0; ++i)
+	    {
+	      c = write(fd, buf, 1);
+	      if (c == -1) break;
+	    }
+
+	  return ENOSPC;
+	}
 
       if (errno == EIO || errno == EAGAIN || errno == EINTR)
 	{
@@ -198,6 +209,9 @@ public int do_write(const char name[], const int fd, void *buf, size_t count)
 
 public int do_lseek(const char name[], const int fd, const off_t offset, const int whence)
 {
+  /* can't seek on stdout */
+  if (options.no_file) return 0;
+
   if (lseek(fd, offset, whence) != offset)
     {
       fprintf(stderr, "\r%s: lseek() failed for `%s': %s\n",
