@@ -1,11 +1,11 @@
 /*
   Licensed under the GNU Public License.
-  Copyright (C) 1998-2002 by Thomas M. Vier, Jr. All Rights Reserved.
+  Copyright (C) 1998-2003 by Thomas M. Vier, Jr. All Rights Reserved.
 
-  wipe v2.0
-  by Tom Vier <nester@users.sourceforge.net>
+  wipe v2.2
+  by Tom Vier <nester@users.sf.net>
 
-  http://wipe.sourceforge.net/
+  http://wipe.sf.net/
 
   wipe is free software.
   See LICENSE for more information.
@@ -28,9 +28,12 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <getopt.h>
 #include <errno.h>
 
+#ifdef HAVE_GETOPT_H
+# include <getopt.h>
+#endif
+
 #include "config.h"
 #include "require.h"
 
@@ -39,11 +42,12 @@
 #include "file.h"
 #include "rand.h"
 #include "text.h"
+#include "wipe.h"
 #include "main.h"
 
 /* global vars */
-char *argvzero;                       /* what wipe was called as, argv[0]        */
-int exit_code = 0;                    /* final exit code for main()              */
+char *argvzero;
+int exit_code;
 
 extern int errno;
 
@@ -57,18 +61,21 @@ public int main(int argc, char **argv)
   extern char *optarg;                /* getopt() stuff                          */
   extern int optind, opterr;          /* getopt() stuff                          */
 
-  opterr = 0;                         /* we'll handle bad options                */
+  opterr=0;                           /* we'll handle bad options                */
 
-  errno = 0;
+  errno=0;
+  exit_code=0;
   argvzero = argv[0];
 
   /* set defaults */
-  options.sectors               = 0;  /* initialize                              */
-  options.sector_size = SECTOR_SIZE;  /* initialize                              */
-  options.chunk_size   = CHUNK_SIZE;  /* initialize                              */
-  options.custom_byte        = 0x00;  /* custom single byte pass                 */
+  options.sectors               = 0;
+  options.sector_size = SECTOR_SIZE;
+  options.chunk_size   = CHUNK_SIZE;
+  options.stdout_size           = 0;
+  options.custom_byte        = 0x00;
   options.verbose               = 1;  /* show percent if >= PERCENT_ENABLE_SIZE  */
   options.recursion             = 0;  /* do not traverse directories             */
+  options.no_file               = 0;  /* use named files                         */
   options.until_full            = 0;  /* don't write until out of space          */
   options.zero                  = 0;  /* don't just zero-out the file            */
   options.force                 = 0;  /* respect file permissions                */
@@ -79,7 +86,7 @@ public int main(int argc, char **argv)
   options.statics               = 1;  /* preform static passes                   */
   options.seclevel              = 1;  /* fast/secure mode                        */
   options.interactive           = 0;  /* don't confirm each file                 */
-  options.random_loop           = 4;  /* default to four random passes           */
+  options.random_loop           = 8;  /* eight random passes per main loop       */
   options.wipe_multiply         = 1;  /* perform wipe loop once                  */
 
 #ifdef SANITY
@@ -91,7 +98,7 @@ public int main(int argc, char **argv)
     }
 #endif
 
-  while ((opt = getopt(argc, argv, "S:C:B:p:b:l::x::XucwsiIhHfFnNdDvVzZrRtTkKaA")) != -1)
+  while ((opt = getopt(argc, argv, "S:C:B:p:b:l::x::XucwsiIhHfFnNdDvVzZrRtTkKaAo::O")) != -1)
     {
       switch (opt)
 	{
@@ -173,7 +180,7 @@ public int main(int argc, char **argv)
 	  break;
 
 	case 'l':  /* set wipe secure level */
-	  if (optarg == 0)
+	  if (!optarg)
 	    options.seclevel = 1;
 	  else
 	    {
@@ -191,7 +198,7 @@ public int main(int argc, char **argv)
 	  break;
 
 	case 'x':  /* perform random passes */
-	  if (optarg == 0)
+	  if (!optarg)
 	    options.random = 1;
 	  else
 	    {
@@ -211,7 +218,7 @@ public int main(int argc, char **argv)
 		  exit(BAD_USAGE);
 		}
 
-	      if (tmp == 0)
+	      if (!tmp)
 		options.random = 0;
 	      else
 		options.random_loop = tmp;
@@ -331,6 +338,32 @@ public int main(int argc, char **argv)
 	  options.until_full = 0;
 	  break;
 
+	case 'o': /* write to stdout */
+	  if (optarg)
+	    {
+	      sscanf(optarg, "%lld", &tmp);
+
+	      if (tmp < 1)
+		{
+		  fprintf(stderr, "%s: bad option: stdout size < 1\n",
+			  argvzero);
+		  exit(BAD_USAGE);
+		}
+
+	      options.stdout_size = tmp;
+	    }
+	  else
+	    options.until_full = 1;
+
+	  options.no_file = 1;
+	  options.verbose = 0;
+	  break;
+
+	case 'O': /* use named files */
+	  options.no_file = 0;
+	  options.stdout_size = 0;
+	  break;
+
 	default:
 	  badopt(optopt);
 	  break;
@@ -339,12 +372,14 @@ public int main(int argc, char **argv)
 
 #ifdef OPTIONTEST
   printf("options are:\n");
-  printf("sectors           = %ld\n", options.sectors);
+  printf("sectors           = %lld\n", options.sectors);
   printf("sector_size       = %ld\n", options.sector_size);
   printf("chunk_size        = %ld\n", options.chunk_size);
+  printf("stdout_size       = %ld\n", options.stdout_size);
   printf("verbose           = %d\n", options.verbose);
   printf("recursion         = %d\n", options.recursion);
   printf("until_full        = %d\n", options.until_full);
+  printf("stdout            = %d\n", options.no_file);
   printf("zero              = %d\n", options.zero);
   printf("force             = %d\n", options.force);
   printf("delete            = %d\n", options.delete);
@@ -357,13 +392,14 @@ public int main(int argc, char **argv)
   printf("interactive       = %d\n", options.interactive);
   printf("random_loop       = %d\n", options.random_loop);
   printf("wipe_multiply     = %d\n\n", options.wipe_multiply);
+  abort();
 #endif
 
 #ifdef FILETEST
   fprintf(stderr, "getopt() parsed %d args\n", optind - 1);
 #endif
 
-  if (optind == argc)
+  if (optind == argc && !options.no_file)
     {
       show_copyright();
       fprintf(stderr, "\nType \'%s -u\' for usage.\n",
@@ -371,14 +407,30 @@ public int main(int argc, char **argv)
       exit(BAD_USAGE);
     }
 
+  /* check for bad combinations */
+  if (options.no_file)
+    {
+      if (options.statics && !options.stdout_size &&
+	  !(options.zero || options.custom_byte))
+	{
+	  fprintf(stderr, "%s: bad option: "
+		  "you must give a stdout length for static passes\n",
+		  argvzero);
+	  exit(BAD_USAGE);
+	}
+    }
+
   if (rand_init())
     {
       fprintf(stderr, "\r%s: rand_init(): fatal error\n", argvzero);
       exit(exit_code);
     }
 
-  /* check access and wipe if ok */
-  while (optind < argc) do_file(argv[optind++]); /*** parse files ***/
+  if (!options.no_file)
+    /* check access and wipe if ok */
+    while (optind < argc) do_file(argv[optind++]); /*** parse files ***/
+  else
+    do_file("stdout"); /*** write to stdout ***/
 
 #ifdef FILETEST
   fprintf(stderr, "\n");
