@@ -24,12 +24,14 @@
 struct opt_s
 {
   unsigned long long sectors;   /* block device sector count                 */
-  size_t sector_size;           /* block device sector size                  */
+  size_t sector_size;           /* sector size                               */
   size_t chunk_size;            /* how big bufsize should be                 */
+  size_t stdout_size;           /* stdout write length                       */
   unsigned char custom_byte;    /* custom overwrite byte                     */
   unsigned int verbose:2;       /* verbose level                             */
   unsigned int recursion:1;     /* traverse directories                      */
   unsigned int until_full:1;    /* write until out of space                  */
+  unsigned int no_file:1;       /* write to stdout                           */
   unsigned int zero:1;          /* zero-out file                             */
   unsigned int lock:1;          /* lock files                                */
   unsigned int force:1;         /* force wipes -- override interaction       */
