@@ -1,6 +1,6 @@
 /*
   Licensed under the GNU Public License.
-  Copyright (C) 1998-2002 by Thomas M. Vier, Jr. All Rights Reserved.
+  Copyright (C) 1998-2004 by Thomas M. Vier, Jr. All Rights Reserved.
 
   wipe is free software.
   See LICENSE for more information.
@@ -27,6 +27,20 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+/* assume either ioctl.h or termio.h has struct winsize */
+#ifdef HAVE_SYS_IOCTL_H
+# include <sys/ioctl.h>
+# define HAVE_WINSIZE
+#endif
+#if HAVE_TERMIO_H
+# include <termio.h>
+# define HAVE_WINSIZE
+#endif
+#if HAVE_TERMIOS_H
+# include <termios.h>
+# define HAVE_WINSIZE
+#endif
+
 #define __USE_GNU
 #include <string.h>
 
@@ -34,6 +48,7 @@
 #include "main.h"
 #include "percent.h"
 #include "file.h"
+#include "str.h"
 
 extern struct opt_s options;
 
@@ -49,22 +64,51 @@ private void percent_line_clear(struct percent_s *p);
 public void percent_init(struct percent_s *p, const char *name, const size_t bufsize, const long int loop)
 {
   int total_passes = STATIC_PASSES;
+  unsigned short scrwid = DEFAULT_SWIDTH;
+#ifdef HAVE_WINSIZE
+  struct winsize ws;
+#endif
 
-  if (!options.verbose)
+  p->display = 0;
+
+  if (options.no_file)
     return;
 
+  if (options.verbose == 1 && bufsize >= PERCENT_ENABLE_SIZE)
+    p->display = 1; /* enable percentage reporting */
+
+  if (options.verbose == 2) /* force percentage reporting */
+    {
+      p->display = 1; p->complete = 0;
+    }
+
+#ifdef HAVE_WINSIZE
+  if (ioctl(1, TIOCGWINSZ, &ws))
+    {/* isn't working */}
+  else
+    scrwid = ws.ws_col;
+#endif
+
   p->complete = 0;
-  p->display = 0;
   p->name = (char *) name;
   p->nlen = strnlen(name, PATH_MAX);
 
+  /* length of ": 100% " is 7 */
+  if (p->nlen + 7 > scrwid)
+    {
+      /* truncate to fit screen */
+      p->nlen = (scrwid - 10) + 3;
+      strncpy(p->name + p->nlen-4, "...", scrwid);
+      p->name[p->nlen+1] = 0;
+    }
+
   if (!(options.zero || options.custom))
     {
       if (options.random)
 	/* two random loops per wipe loop */
 	total_passes += options.random_loop / 2;
 
-      p->increment = 100.0 / (total_passes * options.wipe_multiply);
+      p->increment = 100.0 / ((total_passes * options.wipe_multiply) + 1);
     }
   else
     p->increment = 100.0;
@@ -76,16 +120,11 @@ public void percent_init(struct percent_s *p, const char *name, const size_t buf
       /* percent reporting will have an error margin of +/- chunk_size */
     }
 
-  if (options.verbose == 1 && bufsize >= PERCENT_ENABLE_SIZE)
-    p->display = 1; /* enable percentage reporting */
-
-  if (options.verbose == 2) /* force percentage reporting */
+  if (p->display)
     {
-      p->display = 1; p->complete = 0;
+      percent_line_clear(p);
+      printf(" \r%s: 0%%", name); fflush(stdout); /* display */
     }
-
-  percent_line_clear(p);
-  printf(" \r%s: 0%%", name); fflush(stdout); /* display */
 }
 
 /*
@@ -136,12 +175,12 @@ public void percent_done(struct percent_s *p)
   lastpathlen = p->nlen;
 
   if (!p->display)
-    return;
+    {p->reported = 0; return;}
 
   p_reported = 1;
 
   /*
-     sometimes a double isn't enough decimals to accurately hold
+     sometimes a float isn't enough decimals to accurately hold
      p->increment, so we explicitly print 100% completion.
      otherwise we might end up with (int) p->complete < 100;
      probably 98 or 99.
