@@ -1,6 +1,6 @@
 /*
   Licensed under the GNU Public License.
-  Copyright (C) 1998-2002 by Thomas M. Vier, Jr. All Rights Reserved.
+  Copyright (C) 1998-2003 by Thomas M. Vier, Jr. All Rights Reserved.
 
   wipe is free software.
   See LICENSE for more information.
@@ -20,7 +20,9 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#include <errno.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <fcntl.h>
 #include <sys/types.h>
@@ -79,7 +81,7 @@ public int rand_init(void)
 
 public int prng_seed(void)
 {
-  urand_t seed;
+  u_rand_t seed;
 
   if (do_read(entropy_name, entropyfd, &seed, sizeof(prng_seed)))
     return FAILED;
@@ -89,39 +91,56 @@ public int prng_seed(void)
 }
 
 /*
-  prng_get_urand -- return urand_t PRN
+  prng_get_rand -- return u_rand_t PRN
 */
 
-public urand_t prng_get_urand(void)
+public u_rand_t prng_get_rand(void)
 {
   return randomMT();
 }
 
 /*
   prng_fillbuf -- fills a buffer with pseudo-random values
-                  the buffer must be urand_t aligned and
-		  must be size + sizeof(urand_t) in length
+                  the buffer must be u_rand_t aligned
 */
 
-public void prng_fillbuf(const int seclevel, urand_t *buf, const size_t size)
+public void prng_fillbuf(const int seclevel, u_rand_t *buf, const size_t size)
 {
   int i, ii;
   size_t c;
-  urand_t urand;
-  unsigned char *cbuf, *urandp;
+  u_rand_t rand;
+  unsigned char *cbuf, *randp;
 
-  i=0; c = size - (size % sizeof(urand_t));
-  ii = c / sizeof(urand_t);
+  i=0; c = size - (size % sizeof(u_rand_t));
+  ii = c / sizeof(u_rand_t);
 
   while (i < ii)
     buf[i++] = randomMT();
 
-  urand = randomMT();
-  cbuf = (unsigned char *) ((void *) buf + c);
-  urandp = (unsigned char *) &urand;
+#ifdef SANITY
+  /* check for overflow */
+  if (buf + ii > (buf + size))
+    {fprintf(stderr, "\noverflow in prng_fillbuff()\n"); abort();}
+#endif
 
   i=0; ii = size - c;
 
-  while (i < ii)
-    cbuf[i] = urandp[i++];
+  if (ii)
+    {
+      rand = randomMT();
+      cbuf = (unsigned char *) ((void *) buf + c);
+      randp = (unsigned char *) &rand;
+
+      while (i < ii)
+	{
+	  cbuf[i] = randp[i];
+	  i++;
+	}
+    }
+
+#ifdef SANITY
+  /* overflow? */
+  if ((void *) cbuf + ii > ((void *)buf + size))
+    {fprintf(stderr, "\noverflow in prng_fillbuff()\n"); abort();}
+#endif
 }
