@@ -1,6 +1,6 @@
 /*
   Licensed under the GNU Public License.
-  Copyright (C) 1998-2002 by Thomas M. Vier, Jr. All Rights Reserved.
+  Copyright (C) 1998-2003 by Thomas M. Vier, Jr. All Rights Reserved.
 
   wipe is free software.
   See LICENSE for more information.
@@ -23,13 +23,15 @@
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <dirent.h>
 #include <fcntl.h>
 
+#define __USE_GNU
+#include <string.h>
+
 #include "config.h"
 
 #ifdef STAT_MACROS_BROKEN
@@ -79,12 +81,11 @@ private void pass_pause(void)
 
 /*
   random_pass -- fills file with random bytes
-                 uses it's own commit_pass() code
 */
 
 private int random_pass(struct file_s *f, int passes)
 {
-  int i=0;
+  int i;
 
 #ifdef PAUSE
   fprintf(stderr, "\rentering random_pass()\n");
@@ -97,6 +98,7 @@ private int random_pass(struct file_s *f, int passes)
 
       if (!options.until_full)
 	{
+	  i=0;
 	  while (i++ < f->loop)
 	    {
 	      prng_fillbuf(options.seclevel, f->buf, f->bufsize);
@@ -147,12 +149,13 @@ private int random_pass(struct file_s *f, int passes)
 
 public int write_pass(struct file_s *f, const unsigned char byte)
 {
-  int i=0;
+  int i;
 
   memset(f->buf, byte, f->bufsize);
 
   if (!options.until_full)
     {
+      i=0;
       while (i++ < f->loop)
 	{
 	  if (do_write(f->name, f->fd, f->buf, f->bufsize)) return FAILED;
@@ -193,9 +196,12 @@ public int write_pass(struct file_s *f, const unsigned char byte)
   write3_pass -- over writes file with given bytes
 */
 
-private int write3_pass(struct file_s *f, const unsigned char byte1, const unsigned char byte2, const unsigned char byte3)
+private int write3_pass(struct file_s *f, 
+			const unsigned char byte1, 
+			const unsigned char byte2, 
+			const unsigned char byte3)
 {
-  int i=0, r;
+  int i, r;
   size_t size;
   off_t offset;
   unsigned char bytes[3], *cbuf;
@@ -204,6 +210,7 @@ private int write3_pass(struct file_s *f, const unsigned char byte1, const unsig
   size = f->bufsize - (f->bufsize % 3);
   cbuf = (unsigned char *) f->buf;
 
+  i=0;
   while (i < size)
     {
       cbuf[i++] = byte1;
@@ -276,12 +283,22 @@ private int write3_pass(struct file_s *f, const unsigned char byte1, const unsig
 }
 
 /*
-  destroy -- destroy the file
-             calls subroutines wipe() and zero()
+  destroy_file -- destroy the file
+                  calls wipe() and zero()
 */
 
 public int destroy_file(struct file_s *f)
 {
+  if (options.no_file)
+    {
+      /**** wipe ****/
+      if (wipe(f) == FAILED)
+	return FAILED;
+
+      do_fsync(f->name, f->fd);
+      return 0;
+    }
+
 #ifdef SANITY
   if (!S_ISREG(f->st.st_mode))
     {
@@ -329,20 +346,27 @@ public int destroy_file(struct file_s *f)
     if (do_lock(f))
       goto failure;
 
-  /** rename **/
-  if (options.delete && options.random) 
-    if (wipe_name(f) == FAILED)
-      goto failure;
-
-  /**** wipe ****/
-  if (wipe(f) == FAILED)
-    goto failure;
-
   /*
     if they don't want random passes, or the file won't
     be unlinked, we'll skip the rename
   */
 
+  /** rename **/
+  if (options.delete && options.random)
+    {
+      /* first, just (hopefully) overwrite the name */
+      if (wipe_name(f->real_name, strnlen(f->real_name, sizeof(f->real_name))) == FAILED)
+	goto failure;
+
+      /* now use a rand name */
+      if (wipe_name(f->real_name, 0) == FAILED)
+	goto failure;
+    }
+
+  /**** wipe ****/
+  if (wipe(f) == FAILED)
+    goto failure;
+
   if (options.delete)
     {
       /* destroy file references, unless this is debug build */
@@ -352,7 +376,7 @@ public int destroy_file(struct file_s *f)
 
       /** rename **/
       if (options.random) 
-	if (wipe_name(f) == FAILED)
+	if (wipe_name(f->real_name, 0) == FAILED)
 	  goto failure;
 
 #ifndef DEBUG
@@ -376,7 +400,7 @@ public int destroy_file(struct file_s *f)
 	}
     }
 
-  fsync(f->fd);
+  do_fsync(f->name, f->fd);
 
   do_close(f->name, f->fd);
   return 0;
@@ -414,10 +438,10 @@ private int zero(struct file_s *f)
 
 private int wipe(struct file_s *f)
 {
-  if (f->st.st_size == 0)
+  if (!options.no_file && f->st.st_size == 0)
     return 0; /* no need to write anything */
 
-  if (options.zero || options.custom) return zero(f); /* cute, eh? */
+  if (options.zero || options.custom) return zero(f);
 
   fsetbuf(f);
 
@@ -453,7 +477,7 @@ public int wipe_passes(struct file_s *f)
       if this function is changed, STATIC_PASSES in percent_init()
       must be updated and the above constant must be changed to match
       */
-      fprintf(stderr, "STATIC_PASSES != 27\npercent code broken\n");
+      fprintf(stderr, "\rSTATIC_PASSES != 27\npercent code broken!\n");
       abort();
     }
 
@@ -466,7 +490,7 @@ public int wipe_passes(struct file_s *f)
   while (loop--)
     {
       if (options.random)
-	if (random_pass(f, options.random_loop))
+	if (random_pass(f, options.random_loop / 2))
 	  return FAILED;
 
       /*
@@ -474,6 +498,7 @@ public int wipe_passes(struct file_s *f)
 	Secure Deletion of Data from Magnetic and Solid-State Memory
 
 	http://www.cs.auckland.ac.nz/~pgut001/secure_del.html
+	http://wipe.sourceforge.net/secure_del.html
 
 	thanks, peter!
       */
@@ -518,9 +543,11 @@ public int wipe_passes(struct file_s *f)
 	  if (write3_pass(f, 0xdb, 0x6d, 0xb6)) return FAILED;
 	}
 
+      /* odd rand passes are done last */
       if (options.random)
-	if (random_pass(f, options.random_loop))
+	if (random_pass(f, options.random_loop / 2 + options.random_loop % 2))
 	  return FAILED;
     }
+
   return 0;
 }
